@@ -3,11 +3,6 @@
 
 #include "xh_config.h"
 #include "xh_core.h"
-#ifdef XH_HAVE_DOM
-#include <libxml/parser.h>
-#endif
-
-extern const char indent_string[60];
 
 #define XH_H2X_F_NONE                   0
 #define XH_H2X_F_SIMPLE                 1
@@ -22,47 +17,17 @@ extern const char indent_string[60];
 #define XH_H2X_T_RAW                    16
 #define XH_H2X_T_NOT_NULL               (XH_H2X_T_SCALAR | XH_H2X_T_ARRAY | XH_H2X_T_HASH)
 
-typedef enum {
-    XH_H2X_METHOD_NATIVE = 0,
-    XH_H2X_METHOD_NATIVE_ATTR_MODE,
-    XH_H2X_METHOD_LX
-} xh_h2x_method_t;
+#define XH_H2X_STASH_SIZE               16
 
 typedef struct {
-    xh_h2x_method_t        method;
-
-    /* native options */
-    char                   version[XH_PARAM_LEN];
-    char                   encoding[XH_PARAM_LEN];
-    char                   root[XH_PARAM_LEN];
-    xh_bool_t              xml_decl;
-    xh_bool_t              canonical;
-    char                   content[XH_PARAM_LEN];
-    xh_int_t               indent;
-    void                  *output;
-#ifdef XH_HAVE_DOM
-    xh_bool_t              doc;
-#endif
-    xh_int_t               max_depth;
-
-    /* LX options */
-    char                   attr[XH_PARAM_LEN];
-    size_t                 attr_len;
-    char                   text[XH_PARAM_LEN];
-    xh_bool_t              trim;
-    char                   cdata[XH_PARAM_LEN];
-    char                   comm[XH_PARAM_LEN];
-} xh_h2x_opts_t;
-
-typedef struct {
-    xh_h2x_opts_t          opts;
-    xh_int_t               depth;
-    xh_writer_t           *writer;
-    xh_stack_t             stash;
+    xh_opts_t    opts;
+    xh_int_t     depth;
+    xh_writer_t  writer;
+    xh_stack_t   stash;
 } xh_h2x_ctx_t;
 
 XH_INLINE SV *
-xh_h2x_call_method(SV *obj, GV *method, char *method_name)
+xh_h2x_call_method(SV *obj, GV *method, xh_char_t *method_name)
 {
     int  count;
     SV  *result = &PL_sv_undef;
@@ -179,20 +144,15 @@ xh_h2x_resolve_value(xh_h2x_ctx_t *ctx, SV *value, xh_uint_t *type)
     return value;
 }
 
-xh_h2x_opts_t *xh_h2x_create(void);
-void xh_h2x_destroy(xh_h2x_opts_t *opts);
-xh_bool_t xh_h2x_init_opts(xh_h2x_opts_t *opts);
-void xh_h2x_parse_param(xh_h2x_opts_t *opts, xh_int_t first, I32 ax, I32 items);
-
 SV *xh_h2x(xh_h2x_ctx_t *ctx, SV *hash);
-void xh_h2x_native(xh_h2x_ctx_t *ctx, char *key, I32 key_len, SV *value);
-xh_int_t xh_h2x_native_attr(xh_h2x_ctx_t *ctx, char *key, I32 key_len, SV *value, xh_int_t flag);
+void xh_h2x_native(xh_h2x_ctx_t *ctx, xh_char_t *key, I32 key_len, SV *value);
+xh_int_t xh_h2x_native_attr(xh_h2x_ctx_t *ctx, xh_char_t *key, I32 key_len, SV *value, xh_int_t flag);
 void xh_h2x_lx(xh_h2x_ctx_t *ctx, SV *value, xh_int_t flag);
 
 #ifdef XH_HAVE_DOM
 SV *xh_h2d(xh_h2x_ctx_t *ctx, SV *hash);
-void xh_h2d_native(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, char *key, I32 key_len, SV *value);
-xh_int_t xh_h2d_native_attr(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, char *key, I32 key_len, SV *value, xh_int_t flag);
+void xh_h2d_native(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, xh_char_t *key, I32 key_len, SV *value);
+xh_int_t xh_h2d_native_attr(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, xh_char_t *key, I32 key_len, SV *value, xh_int_t flag);
 void xh_h2d_lx(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, SV *value, xh_int_t flag);
 #endif
 
