@@ -2,12 +2,12 @@
 #include "xh_core.h"
 
 void
-xh_h2x_native(xh_h2x_ctx_t *ctx, char *key, I32 key_len, SV *value)
+xh_h2x_native(xh_h2x_ctx_t *ctx, xh_char_t *key, I32 key_len, SV *value)
 {
     xh_uint_t       type;
     size_t          i, len;
     SV             *item_value;
-    char           *item;
+    xh_char_t      *item;
     I32             item_len;
     xh_sort_hash_t *sorted_hash;
     GV             *method;
@@ -16,7 +16,7 @@ xh_h2x_native(xh_h2x_ctx_t *ctx, char *key, I32 key_len, SV *value)
 
     if (type & XH_H2X_T_BLESSED && (method = gv_fetchmethod_autoload(SvSTASH(value), "iternext", 0)) != NULL) {
         while (1) {
-            item_value = xh_h2x_call_method(value, method, "iternext");
+            item_value = xh_h2x_call_method(value, method, XH_CHAR_CAST "iternext");
             if (!SvOK(item_value)) break;
             (void) xh_h2x_native(ctx, key, key_len, item_value);
             SvREFCNT_dec(item_value);
@@ -25,13 +25,13 @@ xh_h2x_native(xh_h2x_ctx_t *ctx, char *key, I32 key_len, SV *value)
     }
 
     if (type & XH_H2X_T_SCALAR) {
-        xh_xml_write_node(ctx->writer, key, key_len, value, type & XH_H2X_T_RAW);
+        xh_xml_write_node(&ctx->writer, key, key_len, value, type & XH_H2X_T_RAW);
     }
     else if (type & XH_H2X_T_HASH) {
         len = HvUSEDKEYS((HV *) value);
         if (len == 0) goto ADD_EMPTY_NODE;
 
-        xh_xml_write_start_node(ctx->writer, key, key_len);
+        xh_xml_write_start_node(&ctx->writer, key, key_len);
 
         if (len > 1 && ctx->opts.canonical) {
             sorted_hash = xh_sort_hash((HV *) value, len);
@@ -42,12 +42,12 @@ xh_h2x_native(xh_h2x_ctx_t *ctx, char *key, I32 key_len, SV *value)
         }
         else {
             hv_iterinit((HV *) value);
-            while ((item_value = hv_iternextsv((HV *) value, &item, &item_len))) {
+            while ((item_value = hv_iternextsv((HV *) value, (char **) &item, &item_len))) {
                 xh_h2x_native(ctx, item, item_len, item_value);
             }
         }
 
-        xh_xml_write_end_node(ctx->writer, key, key_len);
+        xh_xml_write_end_node(&ctx->writer, key, key_len);
     }
     else if (type & XH_H2X_T_ARRAY) {
         len = av_len((AV *) value) + 1;
@@ -57,7 +57,7 @@ xh_h2x_native(xh_h2x_ctx_t *ctx, char *key, I32 key_len, SV *value)
     }
     else {
 ADD_EMPTY_NODE:
-        xh_xml_write_empty_node(ctx->writer, key, key_len);
+        xh_xml_write_empty_node(&ctx->writer, key, key_len);
     }
 
 FINISH:
@@ -66,12 +66,12 @@ FINISH:
 
 #ifdef XH_HAVE_DOM
 void
-xh_h2d_native(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, char *key, I32 key_len, SV *value)
+xh_h2d_native(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, xh_char_t *key, I32 key_len, SV *value)
 {
     xh_uint_t       type;
     size_t          i, len;
     SV             *item_value;
-    char           *item;
+    xh_char_t      *item;
     I32             item_len;
     xh_sort_hash_t *sorted_hash;
     GV             *method;
@@ -80,7 +80,7 @@ xh_h2d_native(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, char *key, I32 key_len, SV
 
     if (type & XH_H2X_T_BLESSED && (method = gv_fetchmethod_autoload(SvSTASH(value), "iternext", 0)) != NULL) {
         while (1) {
-            item_value = xh_h2x_call_method(value, method, "iternext");
+            item_value = xh_h2x_call_method(value, method, XH_CHAR_CAST "iternext");
             if (!SvOK(item_value)) break;
             (void) xh_h2d_native(ctx, rootNode, key, key_len, item_value);
             SvREFCNT_dec(item_value);
@@ -106,7 +106,7 @@ xh_h2d_native(xh_h2x_ctx_t *ctx, xmlNodePtr rootNode, char *key, I32 key_len, SV
         }
         else {
             hv_iterinit((HV *) value);
-            while ((item_value = hv_iternextsv((HV *) value, &item, &item_len))) {
+            while ((item_value = hv_iternextsv((HV *) value, (char **) &item, &item_len))) {
                 xh_h2d_native(ctx, rootNode, item, item_len, item_value);
             }
         }
