@@ -2,13 +2,13 @@
 #include "xh_core.h"
 
 void
-xh_param_assign_string(char param[], SV *value)
+xh_param_assign_string(xh_char_t param[], SV *value)
 {
-    char *str;
+    xh_char_t *str;
 
     if ( SvOK(value) ) {
-        str = (char *) SvPV_nolen(value);
-        strncpy(param, str, XH_PARAM_LEN);
+        str = XH_CHAR_CAST SvPV_nolen(value);
+        xh_str_copy(param, str, XH_PARAM_LEN);
     }
     else {
         *param = 0;
@@ -16,19 +16,19 @@ xh_param_assign_string(char param[], SV *value)
 }
 
 void
-xh_param_assign_int(char *name, xh_int_t *param, SV *value)
+xh_param_assign_int(xh_char_t *name, xh_int_t *param, SV *value)
 {
-    if ( !SvOK(value) ) {
+    if ( !SvOK(value) )
         croak("Parameter '%s' is undefined", name);
-    }
+
     *param = SvIV(value);
 }
 
 xh_bool_t
 xh_param_assign_bool(SV *value)
 {
-    if ( SvTRUE(value) ) {
+    if ( SvTRUE(value) )
         return TRUE;
-    }
+
     return FALSE;
 }
